﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Metrics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PIbd_22_Kalyshev_Y_V_MotorBoat_Base
{
    internal class SetBoatsGeneric<T>
    where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в списке
        /// </summary>
        public int Count => _places.Count;

        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetBoatsGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="boat">Добавляемый автомобиль</param>
        /// <returns></returns>
        public int Insert(T boat)
        {
            if (_places.Count < _maxCount)
            {
                _places.Add(boat);
                for (int i = 0; i < _places.Count; i++)
                {
                    if (_places[i] == boat)
                    {
                        return i;
                    }
                }
            }
            return -1;
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="boat">Добавляемый автомобиль</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public void Insert(T boat, int position)
        {
            if (_places.Count < _maxCount) 
            {
                if (position < 0 || position > _places.Count)
                {
                    throw new StorageOverflowException();
                }
                _places.Insert(position, boat);
            }
            return;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T Remove(int position)
        {
            if (position < 0 || position > _places.Count)
            {
                throw new BoatNotFoundException(position);
            }
            if (_places[position] == null)
            {
                throw new BoatNotFoundException(position);
            }
            T removed = _places[position];
            _places.RemoveAt(position);
            return removed;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        public T this[int position]
        {
            get
            {
                if (position < 0 || position > _places.Count)
                {
                    return null;
                }
                return _places[position];
            }
            set
            {
                if (position < 0 || position > _places.Count)
                {
                    return;
                }
                _places.Insert(position, value);
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetBoats()
        {
            foreach (var boat in _places)
            {
                if (boat != null)
                {
                    yield return boat;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
